"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskStatus = void 0;
const fs = __importStar(require("fs"));
const url_1 = require("url");
const system_1 = require("../utils/system");
const log_1 = __importDefault(require("../utils/log"));
var TaskStatus;
(function (TaskStatus) {
    TaskStatus[TaskStatus["PENDING"] = 0] = "PENDING";
    TaskStatus[TaskStatus["DONE"] = 1] = "DONE";
    TaskStatus[TaskStatus["DROPPED"] = 2] = "DROPPED";
})(TaskStatus || (exports.TaskStatus = TaskStatus = {}));
function getFileExt(filePath) {
    let ext = "";
    if (filePath.startsWith("http://") || filePath.startsWith("https://")) {
        const urlPath = new url_1.URL(filePath).pathname.slice(1).split("/");
        if (urlPath[urlPath.length - 1].includes(".")) {
            ext = urlPath[urlPath.length - 1].split(".").slice(-1)[0];
        }
    }
    else {
        const filePathArr = filePath.split("/");
        const filename = filePathArr[filePathArr.length - 1];
        ext = filename.includes(".") ? filename.split(".").slice(-1)[0] : "";
    }
    return ext;
}
class FileConcentrator {
    constructor({ taskStatusRecord, outputPath, deleteAfterWritten, ignoreBreakpoints }) {
        this.taskWriteCount = [0];
        this.breakpoints = [];
        this.tasks = [];
        this.lastFinishIndex = -1;
        this.writeSequence = 0;
        this.isCheckingWritableFiles = false;
        this.deleteAfterWritten = false;
        this.ignoreBreakpoints = false;
        this.taskStatusRecords = taskStatusRecord;
        const ext = getFileExt(outputPath);
        this.outputFilename = outputPath.slice(0, -ext.length - 1);
        this.outputFileExt = ext;
        if (deleteAfterWritten) {
            this.deleteAfterWritten = true;
        }
        if (ignoreBreakpoints) {
            this.ignoreBreakpoints = true;
        }
        this.createNextWriteStream();
    }
    createNextWriteStream() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const createWriteStream = () => {
                    this.writeStream = fs.createWriteStream(`${this.outputFilename}_${this.writeSequence}${this.outputFileExt ? `.${this.outputFileExt}` : ""}`);
                    log_1.default.debug(`created new writestream, write sequence ${this.writeSequence}.`);
                };
                if (this.writeStream) {
                    this.writeStream.end(() => {
                        createWriteStream();
                        resolve();
                    });
                }
                else {
                    createWriteStream();
                    resolve();
                }
            }));
        });
    }
    waitStreamWritable(stream) {
        return new Promise((resolve) => {
            stream.once("drain", resolve);
        });
    }
    checkWritableTasks() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCheckingWritableFiles) {
                return;
            }
            this.isCheckingWritableFiles = true;
            log_1.default.debug("check writable tasks start.");
            const writableTasks = [];
            for (let i = this.lastFinishIndex + 1; i <= this.tasks.length; i++) {
                if (!this.tasks[i] && this.taskStatusRecords[i] === TaskStatus.DROPPED) {
                    if (!this.ignoreBreakpoints &&
                        (this.writeSequence !== 0 || this.taskWriteCount[this.writeSequence] !== 0)) {
                        // 文件未下载 但是任务已经被丢弃 忽略空缺 记录文件分割点
                        // 分割点之前还没有写入过则不分割
                        log_1.default.debug(`create new breakpoint at ${i}`);
                        this.breakpoints.push(i);
                    }
                    continue;
                }
                if (!this.tasks[i]) {
                    break;
                }
                writableTasks.push(this.tasks[i]);
                this.taskWriteCount[this.writeSequence]++;
            }
            if (writableTasks.length > 0) {
                yield this.writeFiles(writableTasks);
                if (this.deleteAfterWritten) {
                    for (const task of writableTasks) {
                        fs.unlinkSync(task.filePath);
                    }
                }
            }
            log_1.default.debug("check writable tasks end.");
            this.isCheckingWritableFiles = false;
        });
    }
    writeFiles(tasks) {
        this.lastFinishIndex = tasks[tasks.length - 1].index;
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            let writable = true;
            let counter = 0;
            for (const task of tasks) {
                if (this.breakpoints.includes(task.index - 1)) {
                    log_1.default.debug(`meet write point at ${task.index}.`);
                    this.writeSequence++;
                    yield this.createNextWriteStream();
                }
                writable = this.writeStream.write(fs.readFileSync(task.filePath), () => {
                    counter++;
                    if (counter === tasks.length) {
                        resolve();
                    }
                });
                if (!writable) {
                    // handle back pressure
                    yield this.waitStreamWritable(this.writeStream);
                }
            }
        }));
    }
    addTasks(tasks) {
        for (const task of tasks) {
            this.tasks[task.index] = task;
        }
        this.checkWritableTasks();
    }
    /**
     * 注意：必须在所有任务添加后调用
     */
    waitAllFilesWritten() {
        return __awaiter(this, void 0, void 0, function* () {
            while (this.isCheckingWritableFiles) {
                yield (0, system_1.sleep)(200);
            }
            yield this.checkWritableTasks();
            yield this.closeWriteStream();
            if (this.taskWriteCount[this.writeSequence] === 0) {
                // 如果最后一个文件为空，则删除
                fs.unlinkSync(`${this.outputFilename}_${this.writeSequence}${this.outputFileExt ? `.${this.outputFileExt}` : ""}`);
                this.writeSequence--;
            }
            if (this.writeSequence === 0) {
                // 只输出了一个文件 重命名为原文件名
                fs.renameSync(`${this.outputFilename}_${this.writeSequence}${this.outputFileExt ? `.${this.outputFileExt}` : ""}`, `${this.outputFilename}${this.outputFileExt ? `.${this.outputFileExt}` : ""}`);
            }
        });
    }
    closeWriteStream() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                this.writeStream.end(resolve);
            });
        });
    }
    getOutputFilePaths() {
        const result = [];
        if (this.writeSequence === 0) {
            result.push(`${this.outputFilename}${this.outputFileExt ? `.${this.outputFileExt}` : ""}`);
            return result;
        }
        for (let i = 0; i <= this.writeSequence; i++) {
            result.push(`${this.outputFilename}_${i}${this.outputFileExt ? `.${this.outputFileExt}` : ""}`);
        }
        return result;
    }
}
exports.default = FileConcentrator;
//# sourceMappingURL=file_concentrator.js.map