"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const system_1 = require("../utils/system");
const m3u8_1 = require("../utils/m3u8");
const log_1 = __importDefault(require("../utils/log"));
const downloader_1 = __importDefault(require("./downloader"));
const m3u8_2 = require("./m3u8");
const file_concentrator_1 = require("./file_concentrator");
/**
 * Live Downloader
 */
class LiveDownloader extends downloader_1.default {
    /**
     *
     * @param m3u8Path
     * @param config
     * @param config.threads 线程数量
     */
    constructor(m3u8Path, { threads, output, tempDir, key, verbose, retries, proxy, cookies, headers, keep, noMerge, keepEncryptedChunks, chunkNamingStrategy, cliMode, }) {
        super(m3u8Path, {
            threads: threads || 5,
            output,
            tempDir,
            key,
            verbose,
            retries,
            proxy,
            cookies,
            headers,
            keep,
            noMerge,
            keepEncryptedChunks,
            chunkNamingStrategy,
            cliMode,
        });
        this.finishedInitialChunkUrls = [];
        this.finishedSequenceIds = [];
        this.downloadTasks = [];
        this.allDownloadTasks = [];
        this.runningThreads = 0;
        this.totalCount = 0;
        this.isEnd = false;
        this.isDownloadFinished = false;
        this.isStarted = false;
        this.forceStop = false;
        this.retries = 5;
        if (retries) {
            this.retries = retries;
        }
    }
    refreshM3U8() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.m3u8 = (yield (0, m3u8_1.loadM3U8)({
                    path: this.m3u8Path,
                    retries: this.retries,
                    timeout: this.timeout,
                    initPrimaryKey: this.totalCount,
                }));
            }
            catch (e) {
                if (this.finishedChunkCount > 0) {
                    const responseStatus = (_a = e === null || e === void 0 ? void 0 : e.response) === null || _a === void 0 ? void 0 : _a.status;
                    if (!!responseStatus && responseStatus >= 400 && responseStatus <= 599) {
                        log_1.default.info("M3U8 file is no longer available. Stop downloading.");
                        // Stop downloading
                        this.isEnd = true;
                    }
                }
                else {
                    log_1.default.error("Aborted due to critical error.", e);
                    log_1.default.info(`Your temporary files are located at [${path.resolve(this.tempPath)}]`);
                    this.saveTask();
                    this.emit("critical-error", e);
                }
            }
        });
    }
    download() {
        return __awaiter(this, void 0, void 0, function* () {
            // Record start time to calculate speed.
            this.startedAt = new Date().valueOf();
            // Allocate temporary directory.
            this.tempPath = path.resolve(this.tempPath, "minyami_" + new Date().valueOf());
            if (!fs.existsSync(this.tempPath)) {
                fs.mkdirSync(this.tempPath);
            }
            if (this.cliMode) {
                process.on("SIGINT", () => __awaiter(this, void 0, void 0, function* () {
                    if (!this.forceStop) {
                        log_1.default.info("Ctrl+C pressed, waiting for tasks finished.");
                        this.isEnd = true;
                        this.forceStop = true;
                    }
                    else {
                        log_1.default.info("Force stopped."); // TODO: reject all download promises
                        log_1.default.info(`Your temporary files are located at [${path.resolve(this.tempPath)}]`);
                        this.saveTask();
                        this.emit("finished");
                    }
                }));
            }
            yield this.loadM3U8();
            this.timeout = Math.min(Math.max(20000, this.m3u8.chunks.length * this.m3u8.getChunkLength() * 1000), 60000);
            this.chunkTimeout = Math.min(this.m3u8.getChunkLength() * 1000 * 20, 60000);
            if (this.m3u8.encryptKeys.length > 0) {
                const key = this.m3u8.encryptKeys[0];
                if (key.startsWith("abematv-license")) {
                    log_1.default.info("Site comfirmed: AbemaTV");
                    const parser = yield Promise.resolve().then(() => __importStar(require("./parsers/abema")));
                    parser.default.parse({
                        downloader: this,
                    });
                }
                else {
                    log_1.default.warning(`Site is not supported by Minyami Core. Try common parser.`);
                    try {
                        const parser = yield Promise.resolve().then(() => __importStar(require("./parsers/common")));
                        yield parser.default.parse({
                            downloader: this,
                        });
                    }
                    catch (e) {
                        log_1.default.error("Aborted due to critical error.", e);
                        log_1.default.info(`Your temporary files are located at [${path.resolve(this.tempPath)}]`);
                        this.saveTask();
                        this.emit("critical-error", e);
                    }
                }
            }
            else {
                // Not encrypted
                if (this.m3u8Path.includes("dmc.nico")) {
                    log_1.default.info("Site comfirmed: Niconico.");
                    const parser = yield Promise.resolve().then(() => __importStar(require("./parsers/nicolive")));
                    parser.default.parse({
                        downloader: this,
                    });
                }
                else if (this.m3u8Path.includes("googlevideo")) {
                    // YouTube
                    log_1.default.info("Site comfirmed: YouTube.");
                    const parser = yield Promise.resolve().then(() => __importStar(require("./parsers/youtube")));
                    parser.default.parse({
                        downloader: this,
                    });
                }
                else {
                    log_1.default.warning(`Site is not supported by Minyami Core. Try common parser.`);
                    try {
                        const parser = yield Promise.resolve().then(() => __importStar(require("./parsers/common")));
                        yield parser.default.parse({
                            downloader: this,
                        });
                    }
                    catch (e) {
                        log_1.default.error("Aborted due to critical error.", e);
                        log_1.default.info(`Your temporary files are located at [${path.resolve(this.tempPath)}]`);
                        this.saveTask();
                        this.emit("critical-error", e);
                    }
                }
            }
            this.emit("parsed");
            if (this.verbose) {
                setInterval(() => {
                    log_1.default.debug(`Now running threads: ${this.runningThreads}, finished chunks: ${this.finishedChunkCount}`);
                    this.saveTask();
                }, 3000);
            }
            yield this.checkKeys();
            yield this.cycling();
        });
    }
    cycling() {
        return __awaiter(this, void 0, void 0, function* () {
            while (true) {
                if (this.isEnd) {
                    // 结束下载 进入合并流程
                    break;
                }
                if (this.m3u8.isEnd) {
                    // 到达直播末尾
                    log_1.default.info("Stream ended. Waiting for current tasks finished.");
                    this.isEnd = true;
                }
                const currentPlaylistChunks = [];
                this.m3u8.chunks.forEach((chunk) => {
                    if ((0, m3u8_2.isNormalChunk)(chunk) && !this.finishedSequenceIds.includes(chunk.sequenceId)) {
                        this.finishedSequenceIds.push(chunk.sequenceId);
                        currentPlaylistChunks.push(chunk);
                    }
                    else if (!(0, m3u8_2.isNormalChunk)(chunk) && !this.finishedInitialChunkUrls.includes(chunk.url)) {
                        this.finishedInitialChunkUrls.push(chunk.url);
                        currentPlaylistChunks.push(chunk);
                    }
                });
                log_1.default.debug(`Get ${currentPlaylistChunks.length} new chunk(s).`);
                const newTasks = currentPlaylistChunks.map((chunk, index) => {
                    const id = this.totalCount + index;
                    return {
                        filename: this.onTaskOutputFileNaming(chunk, this.totalCount + index),
                        url: chunk.url,
                        retryCount: 0,
                        chunk,
                        id,
                    };
                });
                // 加入待完成的任务列表 并标记任务初始状态
                this.downloadTasks.push(...newTasks);
                this.allDownloadTasks.push(...newTasks);
                for (const task of newTasks) {
                    this.taskStatusRecord[task.id] = file_concentrator_1.TaskStatus.PENDING;
                }
                this.totalCount += newTasks.length;
                yield this.refreshM3U8();
                yield this.checkKeys();
                if (currentPlaylistChunks.length > 0) {
                    this.checkQueue();
                }
                log_1.default.debug(`Cool down... Wait for next check`);
                yield (0, system_1.sleep)(Math.min(5000, this.m3u8.getChunkLength() * 1000));
            }
        });
    }
    /**
     * Stop downloading for external use
     */
    stopDownload() {
        this.isEnd = true;
    }
    checkQueue() {
        if (this.downloadTasks.length > 0 && this.runningThreads < this.threads) {
            const task = this.downloadTasks.shift();
            this.runningThreads++;
            this.handleTask(task)
                .then(() => {
                this.runningThreads--;
                if (!this.noMerge) {
                    this.fileConcentrator.addTasks([
                        {
                            filePath: (0, m3u8_2.isEncryptedChunk)(task.chunk)
                                ? path.resolve(this.tempPath, `./${task.filename}.decrypt`)
                                : path.resolve(this.tempPath, `./${task.filename}`),
                            index: task.id,
                        },
                    ]);
                    this.taskStatusRecord[task.id] = file_concentrator_1.TaskStatus.DONE;
                }
                const currentChunkInfo = {
                    taskname: task.filename,
                    finishedChunksCount: this.finishedChunkCount,
                    chunkSpeed: this.calculateSpeedByChunk(),
                    ratioSpeed: this.calculateSpeedByRatio(),
                };
                log_1.default.info(`Processing ${currentChunkInfo.taskname} finished. (${currentChunkInfo.finishedChunksCount} chunks downloaded | Avg Speed: ${currentChunkInfo.chunkSpeed} chunks/s or ${currentChunkInfo.ratioSpeed}x)`);
                this.emit("chunk-downloaded", currentChunkInfo);
                this.checkQueue();
            })
                .catch((e) => {
                this.emit("chunk-error", e, task.filename);
                // 重试计数
                if (task.retryCount) {
                    task.retryCount++;
                }
                else {
                    task.retryCount = 1;
                }
                log_1.default.debug(e.message);
                this.runningThreads--;
                if (task.retryCount >= this.retries) {
                    this.taskStatusRecord[task.id] = file_concentrator_1.TaskStatus.DROPPED;
                    log_1.default.warning(`Processing ${task.filename} failed, max retries exceed, drop.`);
                }
                else {
                    log_1.default.warning(`Processing ${task.filename} failed, retry later.`);
                    this.downloadTasks.unshift(task); // 对直播流来说 早速重试比较好
                }
                this.checkQueue();
            });
            this.checkQueue();
        }
        if (this.downloadTasks.length === 0 && this.runningThreads === 0 && this.isEnd && !this.isDownloadFinished) {
            // 结束状态 合并文件
            this.isDownloadFinished = true;
            this.emit("downloaded");
            if (this.noMerge || this.keepTemporaryFiles) {
                this.saveTask();
            }
            if (this.noMerge) {
                log_1.default.info("Skip merging. Please merge video chunks manually.");
                log_1.default.info(`Temporary files are located at ${this.tempPath}`);
                this.emit("finished");
                return;
            }
            log_1.default.info("Merging chunks...");
            this.fileConcentrator
                .waitAllFilesWritten()
                .then(() => __awaiter(this, void 0, void 0, function* () {
                if (!this.keepTemporaryFiles) {
                    log_1.default.info("End of merging.");
                    log_1.default.info("Starting cleaning temporary files.");
                    try {
                        yield (0, system_1.deleteEmptyDirectory)(this.tempPath);
                    }
                    catch (e) {
                        log_1.default.warning(`Fail to delete temporary files, please delete manually or execute "minyami --clean" later.`);
                    }
                }
                const outputPaths = this.fileConcentrator.getOutputFilePaths();
                if (outputPaths.length === 1) {
                    log_1.default.info(`All finished. Please checkout your files at [${path.resolve(outputPaths[0])}]`);
                }
                else {
                    log_1.default.info(`All finished. Please checkout your files at ${outputPaths
                        .map((p) => `[${path.resolve(p)}]`)
                        .join(", ")}.`);
                }
                this.emit("finished");
            }))
                .catch((e) => {
                log_1.default.error("Fail to merge video. Please merge video chunks manually.", e);
                log_1.default.info(`Your temporary files are located at [${path.resolve(this.tempPath)}]`);
                this.saveTask();
                this.emit("critical-error", e);
            });
        }
        if (this.downloadTasks.length === 0 && this.runningThreads === 0 && !this.isEnd) {
            // 空闲状态 一秒后再检查待完成任务列表
            log_1.default.debug("Sleep 1000ms.");
            (0, system_1.sleep)(1000).then(() => {
                this.checkQueue();
            });
        }
    }
    saveTask() {
        const taskInfo = {
            tempPath: this.tempPath,
            m3u8Path: this.m3u8Path,
            outputPath: this.outputPath,
            threads: this.threads,
            cookies: this.cookies,
            headers: this.headers,
            key: this.key,
            verbose: this.verbose,
            startedAt: this.startedAt,
            retries: this.retries,
            timeout: this.timeout,
            proxy: this.proxy,
            downloadTasks: this.allDownloadTasks,
        };
        const savePath = path.resolve(this.tempPath, "./task.json");
        try {
            fs.writeFileSync(savePath, JSON.stringify(taskInfo, null, 2));
        }
        catch (e) {
            log_1.default.warning("Fail to save task info.");
            log_1.default.debug(e);
        }
    }
}
exports.default = LiveDownloader;
//# sourceMappingURL=live.js.map