#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const path = __importStar(require("path"));
const erii_1 = __importDefault(require("erii"));
const archive_1 = __importDefault(require("./core/archive"));
const live_1 = __importDefault(require("./core/live"));
const system_1 = require("./utils/system");
const log_1 = __importDefault(require("./utils/log"));
const time_1 = require("./utils/time");
const agent_1 = __importDefault(require("./utils/agent"));
erii_1.default.setMetaInfo({
    version: JSON.parse(fs.readFileSync(path.resolve(__dirname, "../package.json")).toString())["version"] +
        "\nうめにゃん~ (虎>ω<)",
    name: "Minyami / A lovely video downloader",
});
erii_1.default.bind({
    name: ["help", "h"],
    description: "Show help documentation",
    argument: {
        name: "command",
        description: "Show help of a specified command",
    },
}, (ctx) => {
    ctx.showHelp();
});
erii_1.default.bind({
    name: ["version"],
    description: "Show version",
}, (ctx) => {
    ctx.showVersion();
});
erii_1.default.bind({
    name: ["download", "d"],
    description: "Download video",
    argument: {
        name: "input_path",
        description: "m3u8 file path",
    },
}, (ctx, options) => __awaiter(void 0, void 0, void 0, function* () {
    const path = ctx.getArgument().toString();
    if (options.verbose) {
        log_1.default.enableDebugMode();
    }
    agent_1.default.readProxyConfigurationFromEnv();
    const fileOptions = (0, system_1.readConfigFile)();
    if (Object.keys(fileOptions).length > 0) {
        log_1.default.debug(`Read config file: ${JSON.stringify(fileOptions)}`);
    }
    for (const key of Object.keys(fileOptions)) {
        if (options[key] === undefined) {
            options[key] = fileOptions[key];
        }
    }
    const finalOptions = Object.assign(options, { cliMode: true, logger: log_1.default });
    if (options.live) {
        const downloader = new live_1.default(path, finalOptions);
        downloader.on("finished", () => {
            process.exit();
        });
        downloader.on("critical-error", () => {
            process.exit(1);
        });
        yield downloader.download();
    }
    else {
        const downloader = new archive_1.default(path, finalOptions);
        downloader.on("finished", () => {
            process.exit();
        });
        downloader.on("critical-error", () => {
            process.exit(1);
        });
        yield downloader.init();
        yield downloader.download();
    }
}));
erii_1.default.bind({
    name: ["resume", "r"],
    description: "Resume a download. (Archive)",
    argument: {
        name: "input_path",
        description: "m3u8 file path",
    },
}, (ctx, options) => __awaiter(void 0, void 0, void 0, function* () {
    const path = ctx.getArgument().toString();
    const downloader = new archive_1.default(undefined, {
        cliMode: true,
    });
    downloader.on("finished", () => {
        process.exit();
    });
    downloader.on("critical-error", () => {
        process.exit(1);
    });
    downloader.resume(path);
}));
erii_1.default.bind({
    name: ["clean"],
    description: "Clean cache files",
}, (ctx, options) => {
    if (options.verbose) {
        log_1.default.enableDebugMode();
    }
    const fileOptions = (0, system_1.readConfigFile)();
    if (Object.keys(fileOptions).length > 0) {
        log_1.default.debug(`Read config file: ${JSON.stringify(fileOptions)}`);
    }
    for (const file of fs.readdirSync(path.resolve(fileOptions.tempDir || os.tmpdir()))) {
        if (file.startsWith("minyami_")) {
            (0, system_1.forceDeleteDirectory)(path.resolve(os.tmpdir(), `./${file}`));
        }
    }
    fs.writeFileSync(path.resolve(__dirname, "../tasks.json"), "[]");
});
erii_1.default.addOption({
    name: ["verbose", "debug"],
    description: "Debug output",
});
erii_1.default.addOption({
    name: ["threads"],
    command: "download",
    description: "Threads limit",
    argument: {
        name: "limit",
        description: "(Optional) Limit of threads, defaults to 5",
        validate: "isInt",
    },
});
erii_1.default.addOption({
    name: ["retries"],
    command: "download",
    description: "Retry limit",
    argument: {
        name: "limit",
        description: "(Optional) Limit of retry times",
        validate: "isInt",
    },
});
erii_1.default.addOption({
    name: ["output", "o"],
    command: "download",
    description: "Output path",
    argument: {
        name: "path",
        description: "(Optional) Output file path, defaults to ./output.mkv",
        validate: (outputPath, validateLogger) => {
            if (!outputPath.endsWith(".mkv") && !outputPath.endsWith(".ts")) {
                validateLogger("Output filename must ends with .mkv or .ts.");
                return false;
            }
            if (outputPath.endsWith("mkv")) {
                (0, system_1.exec)("mkvmerge --version")
                    .then(() => {
                    //
                })
                    .catch((e) => {
                    log_1.default.error("Missing dependence: mkvmerge");
                });
            }
            if (path.basename(outputPath).match(/[\*\:|\?<>]/)) {
                validateLogger("Filename should't contain :, |, <, >.");
                return false;
            }
            return true;
        },
    },
});
erii_1.default.addOption({
    name: ["temp-dir"],
    command: "download",
    description: "Temporary file path",
    argument: {
        name: "path",
        description: "(Optional) Temporary file path, defaults to env.TEMP",
    },
});
erii_1.default.addOption({
    name: ["key"],
    command: "download",
    description: "Set key manually (Internal use)",
    argument: {
        name: "key",
        description: "(Optional) Key for decrypt video.",
    },
});
erii_1.default.addOption({
    name: ["cookies"],
    command: "download",
    description: "Cookies used to download",
    argument: {
        name: "cookies",
        description: "",
    },
});
erii_1.default.addOption({
    name: ["headers", "H"],
    command: "download",
    description: "HTTP Header used to download",
    argument: {
        name: "headers",
        description: 'Custom header. eg. "User-Agent: xxxxx". This option will override --cookies.',
    },
});
erii_1.default.addOption({
    name: ["live"],
    command: "download",
    description: "Download live",
});
erii_1.default.addOption({
    name: ["format"],
    command: "download",
    description: "(Optional) Set output format. default: ts",
    argument: {
        name: "format_name",
        description: "Format name. ts or mkv.",
        validate: (formatString) => {
            return ["mkv", "ts"].includes(formatString);
        },
    },
});
erii_1.default.addOption({
    name: ["proxy"],
    command: "download",
    description: "Use the specified HTTP/HTTPS/SOCKS5 proxy",
    argument: {
        name: "proxy-server",
        description: 'Set proxy in [protocol://<host>:<port>] format. eg. --proxy "http://127.0.0.1:1080".',
    },
});
erii_1.default.addOption({
    name: ["slice"],
    command: "download",
    description: "Download specified part of the stream",
    argument: {
        name: "range",
        description: 'Set time range in [<hh:mm:ss>-<hh:mm:ss> format]. eg. --slice "45:00-53:00"',
        validate: (timeString, logger) => {
            if (!timeString.includes("-")) {
                logger(`Invalid time range`);
                return false;
            }
            try {
                const start = timeString.split("-")[0];
                const end = timeString.split("-")[1];
                (0, time_1.timeStringToSeconds)(start);
                (0, time_1.timeStringToSeconds)(end);
                return true;
            }
            catch (e) {
                logger(`Invalid time range`);
                return false;
            }
        },
    },
});
erii_1.default.addOption({
    name: ["no-merge"],
    command: "download",
    description: "Do not merge m3u8 chunks.",
});
erii_1.default.addOption({
    name: ["keep", "k"],
    command: "download",
    description: "Keep temporary files.",
});
erii_1.default.addOption({
    name: ["keep-encrypted-chunks"],
    command: "download",
    description: "Do not delete encrypted chunks after decryption.",
});
erii_1.default.addOption({
    name: ["chunk-naming-strategy"],
    command: "download",
    description: "Temporary file naming strategy. Defaults to 1.",
});
erii_1.default.default(() => {
    erii_1.default.showHelp();
});
erii_1.default.okite();
//# sourceMappingURL=index.js.map