"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadM3U8 = void 0;
const fs = __importStar(require("fs"));
const url_1 = require("url");
const axios_1 = __importDefault(require("axios"));
const m3u8_1 = __importDefault(require("../core/m3u8"));
const agent_1 = __importDefault(require("../utils/agent"));
const log_1 = __importDefault(require("../utils/log"));
function loadM3U8({ path, retries = 1, timeout = 60000, initPrimaryKey }) {
    return __awaiter(this, void 0, void 0, function* () {
        const proxyAgent = agent_1.default.getProxyAgentInstance();
        let m3u8Content, m3u8Path;
        if (path.startsWith("http")) {
            log_1.default.info("Start fetching M3U8 file.");
            while (retries >= 0) {
                const CancelToken = axios_1.default.CancelToken;
                let source = CancelToken.source();
                setTimeout(() => {
                    source && source.cancel();
                    source = null;
                }, timeout);
                try {
                    const response = yield axios_1.default.get(path, {
                        timeout,
                        httpsAgent: proxyAgent ? proxyAgent : undefined,
                        headers: {
                            Host: new url_1.URL(path).host,
                        },
                        cancelToken: source.token,
                    });
                    log_1.default.info("M3U8 file fetched.");
                    m3u8Content = response.data;
                    m3u8Path = response.request.res.responseUrl || path;
                    break;
                }
                catch (e) {
                    log_1.default.warning(`Fail to fetch M3U8 file: [${e.code ||
                        (e.response ? `${e.response.status} ${e.response.statusText}` : undefined) ||
                        e.message ||
                        "UNKNOWN"}]`);
                    log_1.default.warning("If you are downloading a live stream, this may result in a broken output video.");
                    retries--;
                    if (retries >= 0) {
                        log_1.default.info("Try again.");
                    }
                    else {
                        log_1.default.warning("Max retries exceeded. Abort.");
                        throw e;
                    }
                }
                finally {
                    source = null;
                }
            }
            const m3u8 = new m3u8_1.default({ m3u8Content, m3u8Url: m3u8Path, initPrimaryKey });
            return m3u8.parse();
        }
        else {
            // is a local file path
            if (!fs.existsSync(path)) {
                throw new Error(`File '${path}' not found.`);
            }
            log_1.default.info("Loading M3U8 file.");
            m3u8Content = fs.readFileSync(path).toString();
            const m3u8 = new m3u8_1.default({ m3u8Content });
            return m3u8.parse();
        }
    });
}
exports.loadM3U8 = loadM3U8;
//# sourceMappingURL=m3u8.js.map