"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decrypt = exports.requestRaw = exports.download = exports.mergeToTS = exports.mergeToMKV = void 0;
const fs = __importStar(require("fs"));
const url_1 = require("url");
const crypto = __importStar(require("crypto"));
const axios_1 = __importDefault(require("axios"));
const system_1 = require("./system");
const common_1 = require("./common");
const agent_1 = __importDefault(require("./agent"));
/**
 * 合并视频文件
 * @param fileList 文件列表
 * @param output 输出路径
 */
function mergeToMKV(fileList = [], output = "./output.mkv") {
    const outputPath = (0, common_1.getAvailableOutputPath)(output);
    return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
        if (fileList.length === 0) {
            return;
        }
        fileList = fileList.map((file, index) => {
            return index === 0 ? file : `+${file}`;
        });
        const parameters = fileList.concat(["-o", outputPath, "-q"]);
        const tempFilename = `temp_${new Date().valueOf()}.json`;
        fs.writeFileSync(`./${tempFilename}`, JSON.stringify(parameters, null, 2));
        yield (0, system_1.exec)(`mkvmerge @${tempFilename}`);
        fs.unlinkSync(`./${tempFilename}`);
        resolve(outputPath);
    }));
}
exports.mergeToMKV = mergeToMKV;
function mergeToTS(fileList = [], output = "./output.ts") {
    const cliProgress = require("cli-progress");
    const outputPath = (0, common_1.getAvailableOutputPath)(output);
    return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
        if (fileList.length === 0) {
            resolve(outputPath);
        }
        const writeStream = fs.createWriteStream(outputPath);
        const lastIndex = fileList.length - 1;
        const bar = new cliProgress.SingleBar({
            format: "[MINYAMI][MERGING] [{bar}] {percentage}% | ETA: {eta}s | {value}/{total}",
        }, cliProgress.Presets.shades_classic);
        bar.start(fileList.length, 0);
        let i = 0;
        let writable = true;
        write();
        function write() {
            writable = true;
            while (i <= lastIndex && writable) {
                writable = writeStream.write(fs.readFileSync(fileList[i]), () => {
                    if (i > lastIndex) {
                        bar.update(i);
                        bar.stop();
                        writeStream.end();
                        resolve(outputPath);
                    }
                });
                bar.update(i);
                i++;
            }
            if (i <= lastIndex) {
                writeStream.once("drain", () => {
                    write();
                });
            }
        }
    }));
}
exports.mergeToTS = mergeToTS;
/**
 * 下载文件
 * @param url
 * @param path
 */
function download(url, path, options = {}) {
    const CancelToken = axios_1.default.CancelToken;
    let source = CancelToken.source();
    const promise = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            setTimeout(() => {
                source && source.cancel();
                source = null;
            }, options.timeout || 60000);
            const proxyAgentInstance = agent_1.default.getProxyAgentInstance();
            const response = yield (0, axios_1.default)(Object.assign({ url, method: "GET", responseType: "arraybuffer", httpsAgent: proxyAgentInstance ? proxyAgentInstance : undefined, headers: {
                    Host: new url_1.URL(url).host,
                }, cancelToken: source.token }, options));
            if (response.headers["content-length"] &&
                parseInt(response.headers["content-length"]) !== response.data.length) {
                reject(new Error("Bad Response"));
            }
            const tempPath = path + ".t";
            fs.writeFileSync(tempPath, response.data);
            fs.renameSync(tempPath, path);
            resolve();
        }
        catch (e) {
            reject(e);
        }
        finally {
            source = null;
        }
    }));
    return promise;
}
exports.download = download;
/**
 * Raw Request
 * @param url
 * @param proxy
 */
function requestRaw(url, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const proxyAgentInstance = agent_1.default.getProxyAgentInstance();
        return yield (0, axios_1.default)(Object.assign({ url, method: "GET", responseType: "stream", timeout: 60000, httpsAgent: proxyAgentInstance ? proxyAgentInstance : undefined, headers: {
                Host: new url_1.URL(url).host,
            } }, options));
    });
}
exports.requestRaw = requestRaw;
/**
 * 解密文件
 * @param input
 * @param output
 * @param key in hex
 * @param iv in hex
 */
function decrypt(input, output, key, iv, keepEncryptedChunks = false) {
    return new Promise((resolve) => {
        const algorithm = "aes-128-cbc";
        if (key.length !== 32) {
            throw new Error(`Key [${key}] length [${key.length}] or form invalid.`);
        }
        if (iv.length > 32) {
            throw new Error(`IV [${iv}] length [${iv.length}] or form invalid.`);
        }
        if (iv.length % 2 == 1) {
            iv = "0" + iv;
        }
        const keyBuffer = Buffer.alloc(16);
        const ivBuffer = Buffer.alloc(16);
        keyBuffer.write(key, "hex");
        ivBuffer.write(iv, 16 - iv.length / 2, "hex");
        const decipher = crypto.createDecipheriv(algorithm, keyBuffer, ivBuffer);
        const i = fs.createReadStream(input);
        const tempOutput = output + ".t";
        const o = fs.createWriteStream(tempOutput);
        const pipe = i.pipe(decipher).pipe(o);
        pipe.on("close", () => {
            !keepEncryptedChunks && fs.unlinkSync(input);
            fs.renameSync(tempOutput, output);
            resolve();
        });
    });
}
exports.decrypt = decrypt;
//# sourceMappingURL=media.js.map